@@ -49,9 +49,15 @@ Container::Container(QWidget *parent)
     , m_quitTimer(new QTimer(this))
     , m_supportComposite(m_wmHelper->hasComposite())
 {
+    setAccessibleName("Container");
     setWindowFlags(Qt::ToolTip | Qt::WindowTransparentForInput | Qt::WindowDoesNotAcceptFocus);
     setAttribute(Qt::WA_TranslucentBackground);
 
+    if (!qgetenv("WAYLAND_DISPLAY").isEmpty()) {
+        setAttribute(Qt::WA_NativeWindow);
+        windowHandle()->setProperty("_d_dwayland_window-type", "tooltip");
+    }
+
     m_quitTimer->setSingleShot(true);
     m_quitTimer->setInterval(60 * 1000);
 
@@ -103,9 +109,9 @@ void Container::moveToCenter()
     }
 
     if (!displayRect.isValid() || displayRect.isEmpty()) {
-        QDesktopWidget *desktop = QApplication::desktop();
-        const int currentprimary = desktop->screenNumber(QCursor::pos());
-        displayRect = desktop->screenGeometry(currentprimary);
+        const QScreen *screen = qApp->screenAt(QCursor::pos());
+        if (screen)
+            displayRect = screen->geometry();
     }
 
     setGeometry(displayRect);
