@@ -29,6 +29,9 @@
 #include "notifysettings.h"
 #include "notification-center/notifycenterwidget.h"
 #include "dbusdockinterface.h"
+#include "signalbridge.h"
+
+#include <DDesktopServices>
 
 #include <QStringList>
 #include <QVariantMap>
@@ -39,8 +42,6 @@
 #include <QDateTime>
 #include <QGSettings>
 
-#include <DDesktopServices>
-
 #include <algorithm>
 
 #include <com_deepin_daemon_display.h>
@@ -78,12 +79,17 @@ BubbleManager::BubbleManager(AbstractPersistence *persistence, AbstractNotifySet
     initConnections();
     geometryChanged();
 
-    m_notifyCenter->setMaskAlpha(m_appearance->opacity() * 255);
+    m_notifyCenter->setMaskAlpha(static_cast<quint8>(m_appearance->opacity() * 255));
     m_notifyCenter->hide();
     registerAsService();
 
     // 任务栏在左侧时，触屏划入距离需要超过100
     m_slideWidth = (m_dockDeamonInter->position() == OSD::DockPosition::Right) ? 100 : 0;
+
+    connect(m_userInter, &__SessionManager::LockedChanged, this, [ this ] {
+        // 当锁屏状态发生变化时立即隐藏所有通知并插入到通知中心（根据通知的实际情况决定），桌面和锁屏的通知不交叉显示
+        popAllBubblesImmediately();
+    });
 }
 
 BubbleManager::~BubbleManager()
@@ -91,11 +97,13 @@ BubbleManager::~BubbleManager()
     if (!m_bubbleList.isEmpty()) qDeleteAll(m_bubbleList);
 
     m_oldEntities.clear();
+    delete m_notifyCenter;
+    m_notifyCenter = nullptr;
 }
 
 void BubbleManager::CloseNotification(uint id)
 {
-#ifdef QT_DEBUG
+#if defined (QT_DEBUG) && !defined (GTEST)
     QDBusReply<uint> reply = connection().interface()->servicePid(message().service());
     qDebug() << "PID:" << reply.value();//关闭通知的进程
 #endif
@@ -166,6 +174,12 @@ uint BubbleManager::Notify(const QString &appName, uint replacesId,
     }
 #endif
 
+    // 如果display服务无效，无法获取显示器大小，不能正确计算显示位置，则不显示消息通知
+    if (!m_displayInter->isValid()) {
+        qWarning() << "The name com.deepin.daemon.Display is invalid";
+        return 0;
+    }
+
     // 应用通知功能未开启不做处理
     bool enableNotificaion = m_notifySettings->getAppSetting(appName, NotifySettings::ENABELNOTIFICATION).toBool();
 
@@ -244,6 +258,8 @@ void BubbleManager::pushBubble(EntityPtr notify)
     if (notify == nullptr) return;
 
     Bubble *bubble = createBubble(notify);
+    if (!bubble)
+        return;
 
     if (m_bubbleList.size() == BubbleEntities + BubbleOverLap) {
         m_oldEntities.push_front(m_bubbleList.last()->entity());
@@ -264,12 +280,28 @@ void BubbleManager::popBubble(Bubble *bubble)
     m_bubbleList.removeOne(bubble);
 }
 
+void BubbleManager::popAllBubblesImmediately()
+{
+    for (QPointer<Bubble> &bubble : m_bubbleList) {
+        if (bubble) {
+            if (bubble->entity()->isShowInNotifyCenter())
+                m_persistence->addOne(bubble->entity());
+
+            bubble->hide();
+            bubble->close();
+        }
+    }
+
+    m_bubbleList.clear();
+}
+
 void BubbleManager::refreshBubble()
 {
     if (m_bubbleList.size() < BubbleEntities + BubbleOverLap + 1 && !m_oldEntities.isEmpty()) {
         auto notify = m_oldEntities.takeFirst();
         Bubble *bubble = createBubble(notify, BubbleEntities + BubbleOverLap - 1);
-        m_bubbleList.push_back(bubble);
+        if (bubble)
+            m_bubbleList.push_back(bubble);
     }
 }
 
@@ -281,8 +313,8 @@ void BubbleManager::pushAnimation(Bubble *bubble)
 
     while (index < m_bubbleList.size() - 1) {
         index ++;
-        QRect startRect = GetLastStableRect(index - 1);
-        QRect endRect = GetBubbleGeometry(index);
+        QRect startRect = getLastStableRect(index - 1);
+        QRect endRect = getBubbleGeometry(index);
         QPointer<Bubble> item = m_bubbleList.at(index);
         if (item->geometry() != endRect) { //动画中
             startRect = item->geometry();
@@ -300,15 +332,16 @@ void BubbleManager::popAnimation(Bubble *bubble)
     if (index == -1)
         return;
 
-    QRect startRect = GetBubbleGeometry(index);
-    QRect endRect = GetBubbleGeometry(0);
+    QRect startRect = getBubbleGeometry(index);
+    QRect endRect = getBubbleGeometry(0);
 
-    bubble->startMove(startRect, endRect, true);// delete itself
+    if (bubble)
+        bubble->startMove(startRect, endRect, true); // delete itself
 
     while (index < m_bubbleList.size() - 1) {
         index ++;
-        QRect startRect = GetBubbleGeometry(index);
-        QRect endRect = GetBubbleGeometry(index - 1);
+        startRect = getBubbleGeometry(index);
+        endRect = getBubbleGeometry(index - 1);
         QPointer<Bubble> item = m_bubbleList.at(index);
         if (index == BubbleEntities + BubbleOverLap) {
             item->show();
@@ -331,26 +364,19 @@ void BubbleManager::popAnimation(Bubble *bubble)
     }
 }
 
-QRect BubbleManager::GetBubbleGeometry(int index)
+QRect BubbleManager::getBubbleGeometry(int index)
 {
     Q_ASSERT(index >= 0 && index <= BubbleEntities + BubbleOverLap);
 
     QRect rect;
     if (index >= 0 && index <= BubbleEntities - 1) {
-        int y = 0;
-        if (m_dockPos == OSD::Top) {
-            if (m_dockMode == OSD::DockModel::Fashion) {
-                y = m_currentDisplayRect.y() + m_currentDockRect.height() + OSD::DockMargin * 2;
-            } else {
-                y = m_currentDisplayRect.y() + m_currentDockRect.height();
-            }
-        }
+        int y = (m_dockPos == OSD::Top ? m_currentDockRect.bottom() : m_currentDisplayRect.y()); // 多屏时屏幕设置为上下错位，主屏的top可能不是0
         rect.setX(m_currentDisplayRect.x() + (m_currentDisplayRect.width() - OSD::BubbleWidth(OSD::BUBBLEWINDOW)) / 2);
-        rect.setY(y + ScreenPadding + index * (BubbleMargin + OSD::BubbleHeight(OSD::BUBBLEWINDOW)));
+        rect.setY(y + ScreenPadding + index * BubbleMargin + getBubbleHeightBefore(index));
         rect.setWidth(OSD::BubbleWidth(OSD::BUBBLEWINDOW));
         rect.setHeight(OSD::BubbleHeight(OSD::BUBBLEWINDOW));
     } else if (index >= BubbleEntities && index <= BubbleEntities + BubbleOverLap) {
-        rect = GetBubbleGeometry(index - 1);
+        rect = getBubbleGeometry(index - 1);
 
         int x = rect.x() + rect.width() / 20;
         int y = rect.y() + rect.height() / 3;
@@ -366,14 +392,26 @@ QRect BubbleManager::GetBubbleGeometry(int index)
     return rect;
 }
 
-QRect BubbleManager::GetLastStableRect(int index)
+int BubbleManager::getBubbleHeightBefore(const int index)
 {
-    QRect rect = GetBubbleGeometry(0);
+    int totalHeight = 0;
+    for (int i = 0; i < index; i++) {
+        if (m_bubbleList[i]) {
+            totalHeight += m_bubbleList[i]->height();
+        }
+    }
+
+    return totalHeight;
+}
+
+QRect BubbleManager::getLastStableRect(int index)
+{
+    QRect rect = getBubbleGeometry(0);
     for (int i = index - 1; i > 0; --i) {
-        if (i >= m_bubbleList.size() || m_bubbleList.at(i)->geometry() != GetBubbleGeometry(i)) {
+        if (i >= m_bubbleList.size() || m_bubbleList.at(i)->geometry() != getBubbleGeometry(i)) {
             continue;
         }
-        rect = GetBubbleGeometry(i);
+        rect = getBubbleGeometry(i);
     }
 
     return rect;
@@ -423,7 +461,8 @@ QRect BubbleManager::calcDisplayRect()
     for (const auto &screen : screenList) {
         MonitorInter monitor("com.deepin.daemon.Display", screen.path(), QDBusConnection::sessionBus());
         QRect monitorRect(monitor.x(), monitor.y(), monitor.width(), monitor.height());
-        if (monitor.enabled() && monitorRect.contains(m_dockDeamonInter->frontendWindowRect())) {
+        QRect dockRect(m_dockInter->geometry());
+        if (monitor.enabled() && monitorRect.contains(dockRect.center())) {
             displayRect = QRect(monitorRect.x(), monitorRect.y(),
                                 monitorRect.width() / ratio, monitorRect.height() / ratio);
             break;
@@ -509,7 +548,7 @@ QStringList BubbleManager::GetAppList()
     return m_notifySettings->getAppLists();
 }
 
-QDBusVariant BubbleManager::GetAppInfo(const QString id, const uint item)
+QDBusVariant BubbleManager::GetAppInfo(const QString &id, const uint item)
 {
     return QDBusVariant(m_notifySettings->getAppSetting(id, static_cast<NotifySettings::AppConfigurationItem>(item)));
 }
@@ -519,7 +558,7 @@ QDBusVariant BubbleManager::GetSystemInfo(uint item)
     return QDBusVariant(m_notifySettings->getSystemSetting(static_cast<NotifySettings::SystemConfigurationItem>(item)));
 }
 
-void BubbleManager::SetAppInfo(const QString id, const uint item, const QDBusVariant var)
+void BubbleManager::SetAppInfo(const QString &id, const uint item, const QDBusVariant var)
 {
     m_notifySettings->setAppSetting(id, static_cast<NotifySettings::AppConfigurationItem>(item), var.variant());
 }
@@ -543,7 +582,7 @@ void BubbleManager::appInfoChanged(QString action, LauncherItemInfo info)
 
 void BubbleManager::onOpacityChanged(double value)
 {
-    m_notifyCenter->setMaskAlpha(value * 255);
+    m_notifyCenter->setMaskAlpha(static_cast<quint8>(value * 255));
 }
 
 QString BubbleManager::getAllSetting()
@@ -551,7 +590,7 @@ QString BubbleManager::getAllSetting()
     return m_notifySettings->getAllSetings_v1();
 }
 
-void BubbleManager::setAllSetting(const QString settings)
+void BubbleManager::setAllSetting(const QString &settings)
 {
     m_notifySettings->setAllSetting_v1(settings);
 }
@@ -561,7 +600,7 @@ QString BubbleManager::getAppSetting(QString appName)
     return m_notifySettings->getAppSettings_v1(appName);
 }
 
-void BubbleManager::setAppSetting(const QString settings)
+void BubbleManager::setAppSetting(const QString &settings)
 {
     QJsonObject currentObj = QJsonDocument::fromJson(settings.toUtf8()).object();
     m_notifySettings->setAppSetting_v1(settings);
@@ -573,7 +612,7 @@ QString BubbleManager::getSystemSetting()
     return m_notifySettings->getSystemSetings_v1();
 }
 
-void BubbleManager::setSystemSetting(QString settings)
+void BubbleManager::setSystemSetting(const QString &settings)
 {
     m_notifySettings->setSystemSetting_v1(settings);
     Q_EMIT systemSettingChanged(m_notifySettings->getSystemSetings_v1());
@@ -624,7 +663,7 @@ void BubbleManager::bubbleActionInvoked(Bubble *bubble, QString actionId)
 void BubbleManager::updateGeometry()
 {
     foreach (auto item, m_bubbleList) {
-        item->setGeometry(GetBubbleGeometry(item->bubbleIndex()));
+        item->setGeometry(getBubbleGeometry(item->bubbleIndex()));
         item->updateGeometry();
     }
 }
@@ -667,6 +706,8 @@ void BubbleManager::initConnections()
     });
 
     connect(m_appearance, &Appearance::OpacityChanged, this,  &BubbleManager::onOpacityChanged);
+
+    connect(&SignalBridge::ref(), &SignalBridge::actionInvoked, this, &BubbleManager::ActionInvoked);
 }
 
 void BubbleManager::onPrepareForSleep(bool sleep)
@@ -690,6 +731,7 @@ void BubbleManager::geometryChanged()
     m_dockPos = static_cast<OSD::DockPosition>(m_dockDeamonInter->position());
     m_dockMode = m_dockDeamonInter->displayMode();
     m_notifyCenter->updateGeometry(m_currentDisplayRect, m_currentDockRect, m_dockPos, m_dockMode);
+    updateGeometry();
 }
 
 bool BubbleManager::calcReplaceId(EntityPtr notify)
@@ -704,11 +746,11 @@ bool BubbleManager::calcReplaceId(EntityPtr notify)
             Bubble *bubble = m_bubbleList.at(i);
             if (bubble->entity()->replacesId() == notify->replacesId()
                     && bubble->entity()->appName() == notify->appName()) {
-                m_persistence->addOne(m_bubbleList.at(0)->entity());
+                m_persistence->addOne(m_bubbleList.at(i)->entity());
                 if (i != 0) {
-                    bubble->setEntity(m_bubbleList.at(0)->entity());
+                    bubble->setEntity(m_bubbleList.at(i)->entity());
                 }
-                m_bubbleList.at(0)->setEntity(notify);
+                m_bubbleList.at(i)->setEntity(notify);
                 find = true;
             }
         }
@@ -727,39 +769,38 @@ bool BubbleManager::calcReplaceId(EntityPtr notify)
 Bubble *BubbleManager::createBubble(EntityPtr notify, int index)
 {
     Bubble *bubble = new Bubble(nullptr, notify);
-    bubble->setMaskAlpha(m_appearance->opacity() * 255);
+    bubble->setMaskAlpha(static_cast<quint8>(m_appearance->opacity() * 255));
     connect(m_appearance, &Appearance::OpacityChanged, bubble, &Bubble::onOpacityChanged);
     connect(bubble, &Bubble::expired, this, &BubbleManager::bubbleExpired);
     connect(bubble, &Bubble::dismissed, this, &BubbleManager::bubbleDismissed);
     connect(bubble, &Bubble::actionInvoked, this, &BubbleManager::bubbleActionInvoked);
-    connect(bubble, &Bubble::notProcessedYet, this, [ = ](EntityPtr ptr) {
+    connect(bubble, &Bubble::notProcessedYet, this, [ this ](EntityPtr ptr) {
         if (ptr->isShowInNotifyCenter())
             m_persistence->addOne(ptr);
     });
 
     if (index != 0) {
-        QRect startRect = GetBubbleGeometry(BubbleEntities + BubbleOverLap);
-        QRect endRect = GetBubbleGeometry(BubbleEntities + BubbleOverLap - 1);
+        QRect startRect = getBubbleGeometry(BubbleEntities + BubbleOverLap);
+        QRect endRect = getBubbleGeometry(BubbleEntities + BubbleOverLap - 1);
         bubble->setBubbleIndex(BubbleEntities + BubbleOverLap - 1);
         bubble->startMove(startRect, endRect);
     } else {
-        QRect endRect = GetBubbleGeometry(0);
+        QRect endRect = getBubbleGeometry(0);
         QRect startRect = endRect;
-        startRect.setHeight(0);
+        startRect.setHeight(1);
 
-        QPropertyAnimation *ani = new QPropertyAnimation(bubble, "geometry");
+        bubble->setProperty("geometry",0);
+        bubble->show();
+
+        QPropertyAnimation *ani = new QPropertyAnimation(bubble, "geometry", this);
         ani->setStartValue(startRect);
         ani->setEndValue(endRect);
 
         int animationTime = int(endRect.height() * 1.0 / 72 * AnimationTime);
         ani->setDuration(animationTime);
 
-        QTimer::singleShot(0, bubble, [ = ] {
-            bubble->show();
-        });
-
         bubble->setBubbleIndex(0);
-        ani->start();
+        ani->start(QPropertyAnimation::DeleteWhenStopped);
     }
 
     return bubble;
