@@ -1,4 +1,7 @@
 #include "touchscreensetting.h"
+#if (defined QT_DEBUG) && (defined CHECK_ACCESSIBLENAME)
+#include "../common/accessibilitycheckerex.h"
+#endif
 
 #include <DApplication>
 
@@ -26,7 +29,7 @@ int main(int argc, char *argv[])
     QCommandLineParser parser;
     parser.addHelpOption();
     parser.addPositionalArgument("touchscreen",
-                                 "the serial number of touchscreen.",
+                                 "the UUID of touchscreen.",
                                  "[touchscreen]");
     parser.process(app);
 
@@ -44,6 +47,11 @@ int main(int argc, char *argv[])
     }
 
     TouchscreenSetting s(posArguments.first());
+#if (defined QT_DEBUG) && (defined CHECK_ACCESSIBLENAME)
+    AccessibilityCheckerEx checker;
+    checker.setOutputFormat(DAccessibilityChecker::FullFormat);
+    checker.start();
+#endif
     s.show();
     s.moveToCenter();
     return app.exec();
